==> blcr-0.2.0.tar.gz: blcr-0.2.0/examples/pthread_counting/pthread_counting.c <==
/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: pthread_counting.c,v 1.2 2003/11/14 19:55:19 phargrov Exp $
 */


#include <time.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include <errno.h>

#define THREADS 3

static int count = 0;
static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t cond = PTHREAD_COND_INITIALIZER;

static void delay(void)
{
    struct timespec ts = {tv_sec: 1, tv_nsec: 0};
    int rc;

    do {
	rc = nanosleep(&ts, &ts);
    } while ((rc <0) && (errno == EINTR) && (ts.tv_sec || ts.tv_nsec));
}

static void advance(void)
{
    pthread_mutex_lock(&lock);
    count++;
    pthread_cond_broadcast(&cond);
    pthread_mutex_unlock(&lock);
}

static void wait_for(int i)
{
    pthread_mutex_lock(&lock);
    while ((count % THREADS) != i) {
	pthread_cond_wait(&cond, &lock);
    }
    pthread_mutex_unlock(&lock);
}

static void * thread(void * arg)
{
    int id = (int)arg;

    do {
	wait_for(id);
	printf("Thread %d says count = %d\n", id, count);
	delay();
	advance();
    } while (1);

    return NULL;  /* Not reached */
}

int main(void)
{
    pthread_t th[THREADS];
    int i;
	
    printf("Starting main process w/ pid %d\n", (int)getpid());

    // Start one or more threads
    for (i = 1; i < THREADS; ++i) {
	pthread_create(&th[i], NULL, thread, (void *)i);
    }
    thread((void *)0);

    return 0;
}

==> blcr-0.5.0.tar.gz: blcr-0.5.0/examples/pthread_counting/pthread_counting.c <==
/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: pthread_counting.c,v 1.3 2007/01/19 22:02:29 phargrov Exp $
 */


#include <time.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include <errno.h>

#define THREADS 3
#define LIMIT 120

static int count = 0;
static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t cond = PTHREAD_COND_INITIALIZER;

static void delay(void)
{
    struct timespec ts = {tv_sec: 1, tv_nsec: 0};
    int rc;

    do {
	rc = nanosleep(&ts, &ts);
    } while ((rc <0) && (errno == EINTR) && (ts.tv_sec || ts.tv_nsec));
}

static void advance(void)
{
    pthread_mutex_lock(&lock);
    count++;
    pthread_cond_broadcast(&cond);
    pthread_mutex_unlock(&lock);
}

static void wait_for(int i)
{
    pthread_mutex_lock(&lock);
    while ((count % THREADS) != i) {
	pthread_cond_wait(&cond, &lock);
    }
    pthread_mutex_unlock(&lock);
}

static void * thread(void * arg)
{
    int id = (int)arg;
    int local;

    do {
	wait_for(id);
        local = count;
	printf("Thread %d says count = %d\n", id, local);
	delay();
	advance();
    } while (local < LIMIT);

    return NULL;  /* Not reached */
}

int main(void)
{
    pthread_t th[THREADS];
    int i;
	
    printf("Starting main process w/ pid %d\n", (int)getpid());

    // Start one or more threads
    for (i = 1; i < THREADS; ++i) {
	pthread_create(&th[i], NULL, thread, (void *)i);
    }
    thread((void *)0);

    return 0;
}
